"""
mbed SDK
Copyright (c) 2011-2018 ARM Limited

Licensed under the Apache License, Version 2.0 (the 'License');
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an 'AS IS' BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import os
import errno
import logging
import tempfile
import json
from mock import patch, MagicMock, DEFAULT
from io import StringIO

from mbed_lstools.platform_database import PlatformDatabase, DEFAULT_PLATFORM_DB,\
    LOCAL_PLATFORM_DATABASE

try:
    unicode
except NameError:
    unicode = str

class EmptyPlatformDatabaseTests(unittest.TestCase):
    """ Basic test cases with an empty database
    """

    def setUp(self):
        self.base_db_path = os.path.join(tempfile.mkdtemp(), 'base')
        self.base_db = open(self.base_db_path, 'w+b')
        self.base_db.write(b'{}')
        self.base_db.seek(0)
        self.pdb = PlatformDatabase([self.base_db_path])

    def tearDown(self):
        self.base_db.close()

    def test_broken_database_io(self):
        """Verify that the platform database still works without a
        working backing file
        """
        with patch("mbed_os_tools.detect.platform_database.open") as _open:
            _open.side_effect = IOError("Bogus")
            self.pdb = PlatformDatabase([self.base_db_path])
            self.pdb.add("1234", "MYTARGET")
            self.assertEqual(self.pdb.get("1234"), "MYTARGET")

    def test_broken_database_bad_json(self):
        """Verify that the platform database still works without a
        working backing file
        """
        self.base_db.write(b'{}')
        self.base_db.seek(0)
        self.pdb = PlatformDatabase([self.base_db_path])
        self.pdb.add("1234", "MYTARGET")
        self.assertEqual(self.pdb.get("1234"), "MYTARGET")

    def test_broken_database(self):
        """Verify that the platform database correctly reset's its database
        """
        with patch("mbed_os_tools.detect.platform_database.open") as _open,\
             patch("mbed_os_tools.detect.platform_database._older_than_me") as _older:
            _older.return_value = False
            stringio = MagicMock()
            _open.side_effect = (IOError("Bogus"), stringio)
            self.pdb = PlatformDatabase([LOCAL_PLATFORM_DATABASE])
            stringio.__enter__.return_value.write.assert_called_with(
                unicode(json.dumps(DEFAULT_PLATFORM_DB)))
            self.pdb.add("1234", "MYTARGET")
            self.assertEqual(self.pdb.get("1234"), "MYTARGET")

    def test_extra_broken_database(self):
        """Verify that the platform database falls back to the built in database
        even when it can't write to disk
        """
        with patch("mbed_os_tools.detect.platform_database.open") as _open:
            _open.side_effect = IOError("Bogus")
            self.pdb = PlatformDatabase([LOCAL_PLATFORM_DATABASE])
            self.pdb.add("1234", "MYTARGET")
            self.assertEqual(self.pdb.get("1234"), "MYTARGET")

    def test_old_database(self):
        """Verify that the platform database correctly updates's its database
        """
        with patch("mbed_os_tools.detect.platform_database.open") as _open,\
             patch("mbed_os_tools.detect.platform_database.getmtime") as _getmtime:
            file_mock = MagicMock()
            file_mock.read.return_value = ''
            _open.return_value.__enter__.return_value = file_mock
            _getmtime.side_effect = (0, 1000000)
            self.pdb = PlatformDatabase([LOCAL_PLATFORM_DATABASE])
            file_mock.write.assert_called_with(
                unicode(json.dumps(DEFAULT_PLATFORM_DB)))

    def test_bogus_database(self):
        """Basic empty database test
        """
        self.assertEqual(list(self.pdb.items()), [])
        self.assertEqual(list(self.pdb.all_ids()), [])
        self.assertEqual(self.pdb.get('Also_Junk', None), None)

    def test_add(self):
        """Test that what was added can later be queried
        """
        self.assertEqual(self.pdb.get('4753', None), None)
        self.pdb.add('4753', 'Test_Platform', permanent=False)
        self.assertEqual(self.pdb.get('4753', None), 'Test_Platform')

    def test_remove(self):
        """Test that once something is removed it no longer shows up when queried
        """
        self.assertEqual(self.pdb.get('4753', None), None)
        self.pdb.add('4753', 'Test_Platform', permanent=False)
        self.assertEqual(self.pdb.get('4753', None), 'Test_Platform')
        self.assertEqual(self.pdb.remove('4753', permanent=False), 'Test_Platform')
        self.assertEqual(self.pdb.get('4753', None), None)

    def test_bogus_add(self):
        """Test that add requires properly formatted platform ids
        """
        self.assertEqual(self.pdb.get('NOTVALID', None), None)
        with self.assertRaises(ValueError):
            self.pdb.add('NOTVALID', 'Test_Platform', permanent=False)

    def test_bogus_remove(self):
        """Test that removing a not present platform does nothing
        """
        self.assertEqual(self.pdb.get('NOTVALID', None), None)
        self.assertEqual(self.pdb.remove('NOTVALID', permanent=False), None)

    def test_simplify_verbose_data(self):
        """Test that fetching a verbose entry without verbose data correctly
        returns just the 'platform_name'
        """
        platform_data = {
            'platform_name': 'VALID',
            'other_data': 'data'
        }
        self.pdb.add('1337', platform_data, permanent=False)
        self.assertEqual(self.pdb.get('1337', verbose_data=True), platform_data)
        self.assertEqual(self.pdb.get('1337'), platform_data['platform_name'])

class OverriddenPlatformDatabaseTests(unittest.TestCase):
    """ Test that for one database overriding another
    """

    def setUp(self):
        self.temp_dir = tempfile.mkdtemp()
        self.base_db_path = os.path.join(self.temp_dir, 'base')
        self.base_db = open(self.base_db_path, 'w+b')
        self.base_db.write(json.dumps(dict([('0123', 'Base_Platform')])).
                           encode('utf-8'))
        self.base_db.seek(0)
        self.overriding_db_path = os.path.join(self.temp_dir, 'overriding')
        self.overriding_db = open(self.overriding_db_path, 'w+b')
        self.overriding_db.write(b'{}')
        self.overriding_db.seek(0)
        self.pdb = PlatformDatabase([self.overriding_db_path, self.base_db_path],
                                    primary_database=self.overriding_db_path)
        self.base_db.seek(0)
        self.overriding_db.seek(0)

    def tearDown(self):
        self.base_db.close()
        self.overriding_db.close()

    def assertBaseUnchanged(self):
        """Assert that the base database has not changed
        """
        self.base_db.seek(0)
        self.assertEqual(self.base_db.read(),
                         json.dumps(dict([('0123', 'Base_Platform')]))
                         .encode('utf-8'))

    def assertOverrideUnchanged(self):
        """Assert that the override database has not changed
        """
        self.overriding_db.seek(0)
        self.assertEqual(self.overriding_db.read(), b'{}')

    def test_basline(self):
        """Sanity check that the base database does what we expect
        """
        self.assertEqual(list(self.pdb.items()), [('0123', 'Base_Platform')])
        self.assertEqual(list(self.pdb.all_ids()), ['0123'])

    def test_add_non_override(self):
        """Check that adding keys goes to the Override database
        """
        self.pdb.add('1234', 'Another_Platform')
        self.assertEqual(list(self.pdb.items()), [('1234', 'Another_Platform'), ('0123', 'Base_Platform')])
        self.assertEqual(set(self.pdb.all_ids()), set(['0123', '1234']))
        self.assertBaseUnchanged()

    def test_load_override(self):
        """Check that adding a platform goes to the Override database and
        you can no longer query for the base database definition and
        that the override database was not written to disk
        """
        self.overriding_db.write(json.dumps(dict([('0123', 'Overriding_Platform')])).
                                 encode('utf-8'))
        self.overriding_db.seek(0)
        self.pdb = PlatformDatabase([self.overriding_db_path, self.base_db_path],
                                    primary_database=self.overriding_db_path)
        self.assertIn(('0123', 'Overriding_Platform'), list(self.pdb.items()))
        self.assertEqual(set(self.pdb.all_ids()), set(['0123']))
        self.assertEqual(self.pdb.get('0123'), 'Overriding_Platform')
        self.assertBaseUnchanged()

    def test_add_override_permanent(self):
        """Check that adding a platform goes to the Override database and
        you can no longer query for the base database definition and
        that the override database was written to disk
        """
        self.pdb.add('0123', 'Overriding_Platform', permanent=True)
        self.assertIn(('0123', 'Overriding_Platform'), list(self.pdb.items()))
        self.assertEqual(set(self.pdb.all_ids()), set(['0123']))
        self.assertEqual(self.pdb.get('0123'), 'Overriding_Platform')
        self.overriding_db.seek(0)
        self.assertEqual(self.overriding_db.read(),
                         json.dumps(dict([('daplink', dict([('0123', 'Overriding_Platform')]))]))
                         .encode('utf-8'))
        self.assertBaseUnchanged()

    def test_remove_override(self):
        """Check that removing a platform from the Override database allows you to query
        the original base database definition and that
        that the override database was not written to disk
        """
        self.pdb.add('0123', 'Overriding_Platform')
        self.assertIn(('0123', 'Overriding_Platform'), list(self.pdb.items()))
        self.assertEqual(set(self.pdb.all_ids()), set(['0123']))
        self.assertEqual(self.pdb.get('0123'), 'Overriding_Platform')
        self.assertEqual(self.pdb.remove('0123'), 'Overriding_Platform')
        self.assertEqual(self.pdb.get('0123'), 'Base_Platform')
        self.assertOverrideUnchanged()
        self.assertBaseUnchanged()

    def test_remove_from_base(self):
        """Check that removing a platform from the base database no longer allows you to query
        the original base database definition and that that the base database
        was not written to disk
        """
        self.assertEqual(self.pdb.remove('0123'), 'Base_Platform')
        self.assertEqual(self.pdb.get('0123'), None)
        self.assertOverrideUnchanged()
        self.assertBaseUnchanged()

    def test_remove_from_base_permanent(self):
        """Check that removing a platform from the base database no longer allows you to query
        the original base database definition and that that the base database
        was not modified on disk
        """
        self.assertEqual(self.pdb.remove('0123', permanent=True), 'Base_Platform')
        self.assertEqual(self.pdb.get('0123'), None)
        self.assertBaseUnchanged()

class InternalLockingChecks(unittest.TestCase):

    def setUp(self):
        self.mocked_lock = patch('mbed_os_tools.detect.platform_database.InterProcessLock', spec=True).start()
        self.acquire = self.mocked_lock.return_value.acquire
        self.release = self.mocked_lock.return_value.release

        self.base_db_path = os.path.join(tempfile.mkdtemp(), 'base')
        self.base_db = open(self.base_db_path, 'w+b')
        self.base_db.write(b'{}')
        self.base_db.seek(0)
        self.pdb = PlatformDatabase([self.base_db_path])
        self.addCleanup(patch.stopall)

    def tearDown(self):
        self.base_db.close()

    def test_no_update(self):
        """Test that no locks are used when no modifications are specified
        """
        self.pdb.add('7155', 'Junk')
        self.acquire.assert_not_called()
        self.release.assert_not_called()

    def test_update(self):
        """Test that locks are used when modifications are specified
        """
        self.pdb.add('7155', 'Junk', permanent=True)
        assert self.acquire.called, 'Lock acquire should have been called'
        assert self.release.called

    def test_update_fail_acquire(self):
        """Test that the backing file is not updated when lock acquisition fails
        """
        self.acquire.return_value = False
        self.pdb.add('7155', 'Junk', permanent=True)
        assert self.acquire.called, 'Lock acquire should have been called'
        self.base_db.seek(0)
        self.assertEqual(self.base_db.read(), b'{}')

    def test_update_ambiguous(self):
        """Test that the backing file is not updated when lock acquisition fails
        """
        self.pdb._prim_db = None
        self.pdb.add('7155', 'Junk', permanent=True)
        self.acquire.assert_not_called()
        self.release.assert_not_called()
        self.assertEqual(self.base_db.read(), b'{}')



"""
mbed SDK
Copyright (c) 2011-2018 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

"""! @package mbed-ls

mbed-ls package is a set of tools inherited from mbed-lmtools used to detect
mbed enabled devices on host (Windows, Linux and MacOS).

mbed-lstools is a Python module that detects and lists mbed-enabled devices
connected to the host computer. It will be delivered as a redistributable
Python module (package) and command line tool.

Currently supported operating system:
* Windows 7.
* Ubuntu.
* Mac OS X (Darwin).

The stand-alone mbed-lstools Python package is still under development,
but it's already delivered as part of the mbed SDK's test suite and a command
line tool (see below).
"""

from .main import mbedls_main
from .main import create



"""
mbed SDK
Copyright (c) 2011-2018 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import os
import sys
import errno
import logging

import platform
from mbed_lstools.main import create
from mbed_lstools.main import mbed_os_support
from mbed_lstools.main import mbed_lstools_os_info


class DetectOSTestCase(unittest.TestCase):
    """ Test cases for host OS related functionality. Helpful during porting
    """

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_porting_mbed_lstools_os_info(self):
        self.assertNotEqual(None, mbed_lstools_os_info())

    def test_porting_mbed_os_support(self):
        self.assertNotEqual(None, mbed_os_support())

    def test_porting_create(self):
        self.assertNotEqual(None, create())

    def test_supported_os_name(self):
        os_names = ['Windows7', 'Ubuntu', 'LinuxGeneric', 'Darwin']
        self.assertIn(mbed_os_support(), os_names)

    def test_detect_os_support_ext(self):
        os_info = (os.name,
                   platform.system(),
                   platform.release(),
                   platform.version(),
                   sys.platform)

        self.assertEqual(os_info, mbed_lstools_os_info())


if __name__ == '__main__':
    unittest.main()



"""
mbed SDK
Copyright (c) 2011-2018 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import os
import copy
from mock import patch, mock_open, DEFAULT

from mbed_lstools.lstools_base import MbedLsToolsBase

TEST_DATA_PATH = 'test_data'

class DummyLsTools(MbedLsToolsBase):
    return_value = []
    def find_candidates(self):
        return self.return_value

try:
    basestring
except NameError:
    # Python 3
    basestring = str


def get_case_insensitive_path(path, file_name):
    for entry in os.listdir(path):
        if entry.lower() == file_name.lower():
            return os.path.join(path, entry)

    raise Exception('No matching file for %s found in $s' % (file_name, path))


class PlatformDetectionTestCase(unittest.TestCase):
    """ Basic test cases checking trivial asserts
    """

    def setUp(self):
        self.base = DummyLsTools()

    def tearDown(self):
        pass

    def run_test(self, test_data_case, candidate_data, expected_data):
        # Add necessary candidate data
        candidate_data['mount_point'] = 'dummy_mount_point'

        # Find the test data in the test_data folder
        test_script_path = os.path.dirname(os.path.abspath(__file__))
        test_data_path = os.path.join(test_script_path, TEST_DATA_PATH)
        test_data_cases = os.listdir(test_data_path)
        self.assertTrue(test_data_case in test_data_cases, 'Expected %s to be present in %s folder' % (test_data_case, test_data_path))
        test_data_case_path = os.path.join(test_data_path, test_data_case)

        # NOTE a limitation of this mocked test is that it only allows mocking of one directory level.
        # This is enough at the moment because all firmwares seem to use a flat file structure.
        # If this changes in the future, this mocking framework can be extended to support this.

        test_data_case_file_names = os.listdir(test_data_case_path)
        mocked_open_file_paths = [os.path.join(candidate_data['mount_point'], file_name ) for file_name in test_data_case_file_names]

        # Setup all the mocks
        self.base.return_value = [candidate_data]

        def do_open(path, mode='r'):
            file_name = os.path.basename(path)
            try:
                with open(get_case_insensitive_path(test_data_case_path, file_name), 'r') as test_data_file:
                    test_data_file_data = test_data_file.read()
            except OSError:
                raise OSError("(mocked open) No such file or directory: '%s'" % (path))

            file_object = mock_open(read_data=test_data_file_data).return_value
            file_object.__iter__.return_value = test_data_file_data.splitlines(True)
            return file_object

        with patch("mbed_lstools.lstools_base.MbedLsToolsBase.mount_point_ready") as _mpr,\
             patch('mbed_os_tools.detect.lstools_base.open', do_open) as _,\
             patch("mbed_os_tools.detect.lstools_base.listdir") as _listdir:
            _mpr.return_value = True
            _listdir.return_value = test_data_case_file_names
            results = self.base.list_mbeds(read_details_txt=True)

            # There should only ever be one result
            self.assertEqual(len(results), 1)
            actual = results[0]

            expected_keys = list(expected_data.keys())
            actual_keys = list(actual.keys())
            differing_map = {}
            for key in expected_keys:
                actual_value = actual.get(key)
                if actual_value != expected_data[key]:
                    differing_map[key] = (actual_value, expected_data[key])


            if differing_map:
                differing_string = ''
                for differing_key in sorted(list(differing_map.keys())):
                    actual, expected = differing_map[differing_key]
                    differing_string += '    "%s": "%s" (expected "%s")\n' % (differing_key, actual, expected)

                assert_string = 'Expected data mismatch:\n\n{\n%s}' % (differing_string)
                self.assertTrue(False, assert_string)



    def test_efm32pg_stk3401_jlink(self):
        self.run_test('efm32pg_stk3401_jlink', {
            'target_id_usb_id': u'000440074453',
            'vendor_id': '1366',
            'product_id': '1015'
        }, {
            'platform_name': 'EFM32PG_STK3401',
            'device_type': 'jlink',
            'target_id': '2035022D000122D5D475113A',
            'target_id_usb_id': '000440074453',
            'target_id_mbed_htm': '2035022D000122D5D475113A'
        })

    def test_lpc1768(self):
        self.run_test('lpc1768', {
            'target_id_usb_id': u'101000000000000000000002F7F20DF3',
            'vendor_id': '0d28',
            'product_id': '0204'
        }, {
            'platform_name': 'LPC1768',
            'device_type': 'daplink',
            'target_id': '101000000000000000000002F7F20DF3d51e6be5ac41795761dc44148e3b7000',
            'target_id_usb_id': '101000000000000000000002F7F20DF3',
            'target_id_mbed_htm': '101000000000000000000002F7F20DF3d51e6be5ac41795761dc44148e3b7000'
        })

    def test_nucleo_f411re_stlink(self):
        self.run_test('nucleo_f411re_stlink', {
            'target_id_usb_id': u'0671FF554856805087112815',
            'vendor_id': '0483',
            'product_id': '374b'
        }, {
            'platform_name': 'NUCLEO_F411RE',
            'device_type': 'stlink',
            'target_id': '07400221076061193824F764',
            'target_id_usb_id': '0671FF554856805087112815',
            'target_id_mbed_htm': '07400221076061193824F764'
        })

    def test_nrf51_microbit(self):
        self.run_test('nrf51_microbit', {
            'target_id_usb_id': u'9900007031324e45000f9019000000340000000097969901',
            'vendor_id': '0d28',
            'product_id': '0204'
        }, {
            'platform_name': 'NRF51_MICROBIT',
            'device_type': 'daplink',
            'target_id': '9900007031324e45000f9019000000340000000097969901',
            'target_id_usb_id': '9900007031324e45000f9019000000340000000097969901'
        })

    def test_k64f_daplink(self):
        self.run_test('k64f_daplink', {
            'target_id_usb_id': u'0240000032044e45000a700a997b00356781000097969900',
            'vendor_id': '0d28',
            'product_id': '0204'
        }, {
            'platform_name': 'K64F',
            'device_type': 'daplink',
            'target_id': '0240000032044e45000a700a997b00356781000097969900',
            'target_id_usb_id': '0240000032044e45000a700a997b00356781000097969900',
            'target_id_mbed_htm': '0240000032044e45000a700a997b00356781000097969900'
        })

    def test_nrf52_dk_daplink(self):
        self.run_test('nrf52_dk_daplink', {
            'target_id_usb_id': u'110100004420312043574641323032203233303397969903',
            'vendor_id': '0d28',
            'product_id': '0204'
        }, {
            'platform_name': 'NRF52_DK',
            'device_type': 'daplink',
            'target_id': '110100004420312043574641323032203233303397969903',
            'target_id_usb_id': '110100004420312043574641323032203233303397969903',
            'target_id_mbed_htm': '110100004420312043574641323032203233303397969903'
        })

    def test_nrf52_dk_jlink(self):
        self.run_test('nrf52_dk_jlink', {
            'target_id_usb_id': u'000682546728',
            'vendor_id': '1366',
            'product_id': '1015'
        }, {
            'platform_name': 'NRF52_DK',
            'device_type': 'jlink',
            'target_id': '000682546728',
            'target_id_usb_id': '000682546728'
        })



if __name__ == '__main__':
    unittest.main()




"""
mbed SDK
Copyright (c) 2011-2018 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import sys
import json
import argparse
from collections import defaultdict

# Make sure that any global generic setup is run
from . import lstools_base
from mbed_os_tools.detect.main import (
    create,
    mbed_os_support,
    mbed_lstools_os_info,
    mock_platform
)

import logging
logger = logging.getLogger("mbedls.main")
logger.addHandler(logging.NullHandler())
del logging

def get_version():
    """! Get mbed-ls Python module version string """
    import pkg_resources  # part of setuptools
    return pkg_resources.require("mbed-ls")[0].version

def print_version(mbeds, args):
    print(get_version())

def print_mbeds(mbeds, args, simple):
    devices = mbeds.list_mbeds(unique_names=True, read_details_txt=True)
    if devices:
        from prettytable import PrettyTable, HEADER
        columns = ['platform_name', 'platform_name_unique', 'mount_point',
                    'serial_port', 'target_id', 'daplink_version']
        pt = PrettyTable(columns, junction_char="|", hrules=HEADER)
        pt.align = 'l'
        for d in devices:
            pt.add_row([d.get(col, None) or 'unknown' for col in columns])
        print(pt.get_string(border=not simple, header=not simple,
                            padding_width=1, sortby='platform_name_unique'))

def print_table(mbeds, args):
    return print_mbeds(mbeds, args, False)

def print_simple(mbeds, args):
    return print_mbeds(mbeds, args, True)

def list_platforms(mbeds, args):
    print(mbeds.list_manufacture_ids())

def mbeds_as_json(mbeds, args):
    print(json.dumps(mbeds.list_mbeds(unique_names=True,
                                      read_details_txt=True),
                     indent=4, sort_keys=True))

def json_by_target_id(mbeds, args):
    print(json.dumps({m['target_id']: m for m
                      in mbeds.list_mbeds(unique_names=True,
                                          read_details_txt=True)},
                     indent=4, sort_keys=True))

def json_platforms(mbeds, args):
    platforms = set()
    for d in mbeds.list_mbeds():
        platforms |= set([d['platform_name']])
    print(json.dumps(list(platforms), indent=4, sort_keys=True))

def json_platforms_ext(mbeds, args):
    platforms = defaultdict(lambda: 0)
    for d in mbeds.list_mbeds():
        platforms[d['platform_name']] += 1
    print(json.dumps(platforms, indent=4, sort_keys=True))

def parse_cli(to_parse):
    """! Parse the command line

    @return Retrun a namespace that contains:
     * command - python function to run
     * skip_retarget - bool indicting to skip retargeting
     * list_unmounted - list boards that are not mounted
     * debug - turn on debug logging
    """
    parser = argparse.ArgumentParser()
    parser.set_defaults(command=print_table)

    commands = parser.add_argument_group('sub commands')\
                     .add_mutually_exclusive_group()
    commands.add_argument(
        '-s', '--simple', dest='command', action='store_const',
        const=print_simple,
        help='list attached targets without column headers and borders')
    commands.add_argument(
        '-j', '--json', dest='command', action='store_const',
        const=mbeds_as_json,
        help='list attached targets with detailed information in JSON format')
    commands.add_argument(
        '-J', '--json-by-target-id', dest='command', action='store_const',
        const=json_by_target_id,
        help='map attached targets from their target ID to their detailed '
        'information in JSON format')
    commands.add_argument(
        '-p', '--json-platforms', dest='command', action='store_const',
        const=json_platforms,
        help='list attached platform names in JSON format.')
    commands.add_argument(
        '-P', '--json-platforms-ext', dest='command', action='store_const',
        const=json_platforms_ext,
        help='map attached platform names to the number of attached boards in '
        'JSON format')
    commands.add_argument(
        '-l', '--list', dest='command', action='store_const',
        const=list_platforms,
        help='list all target IDs and their corresponding platform names '
        'understood by mbed-ls')
    commands.add_argument(
        '--version', dest='command', action='store_const', const=print_version,
        help='print package version and exit')
    commands.add_argument(
        '-m', '--mock', metavar='ID:NAME',
        help='substitute or create a target ID to platform name mapping used'
        'when invoking mbedls in the current directory')

    parser.add_argument(
        '--skip-retarget', dest='skip_retarget', default=False,
        action="store_true",
        help='skip parsing and interpretation of the re-target file,'
        ' `./mbedls.json`')
    parser.add_argument(
        '-u', '--list-unmounted', dest='list_unmounted', default=False,
        action='store_true',
        help='list mbeds, regardless of whether they are mounted or not')
    parser.add_argument(
        '-d', '--debug', dest='debug', default=False, action="store_true",
        help='outputs extra debug information useful when creating issues!')

    args = parser.parse_args(to_parse)
    if args.mock:
        args.command = mock_platform
    return args

def start_logging():
    try:
        import colorlog
        colorlog.basicConfig(
            format='%(log_color)s%(levelname)s%(reset)s:%(name)s:%(message)s')
    except ImportError:
        import logging
        logging.basicConfig()
        del logging

def mbedls_main():
    """! Function used to drive CLI (command line interface) application
    @return Function exits with success code
    """
    start_logging()

    args = parse_cli(sys.argv[1:])

    import logging
    root_logger = logging.getLogger("mbedls")
    if args.debug:
        root_logger.setLevel(logging.DEBUG)
    else:
        root_logger.setLevel(logging.INFO)
    del logging
    logger.debug("mbed-ls ver. %s", get_version())
    logger.debug("host: %s",  str(mbed_lstools_os_info()))

    mbeds = create(skip_retarget=args.skip_retarget,
                   list_unmounted=args.list_unmounted,
                   force_mock=args.command is mock_platform)

    if mbeds is None:
        logger.critical('This platform is not supported! Pull requests welcome at github.com/ARMmbed/mbed-ls')
        sys.exit(1)

    ret_code = args.command(mbeds, args)
    if not ret_code:
        ret_code = 0

    logger.debug("Return code: %d", ret_code)

    sys.exit(ret_code)



"""
mbed SDK
Copyright (c) 2011-2018 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import os
import errno
import logging
import re
import json
from io import StringIO
from mock import patch, mock_open, DEFAULT
from copy import deepcopy

from mbed_lstools.lstools_base import MbedLsToolsBase, FSInteraction

class DummyLsTools(MbedLsToolsBase):
    return_value = []
    def find_candidates(self):
        return self.return_value

try:
    basestring
except NameError:
    # Python 3
    basestring = str

class BasicTestCase(unittest.TestCase):
    """ Basic test cases checking trivial asserts
    """

    def setUp(self):
        self.base = DummyLsTools(force_mock=True)

    def tearDown(self):
        pass

    def test_list_mbeds_valid_platform(self):
        self.base.return_value = [{'mount_point': 'dummy_mount_point',
                                   'target_id_usb_id': u'0240DEADBEEF',
                                   'serial_port': "dummy_serial_port"},
                                  {'mount_point': None,
                                   'target_id_usb_id': '00000000000',
                                   'serial_port': 'not_valid'}]
        with patch("mbed_lstools.lstools_base.MbedLsToolsBase._read_htm_ids") as _read_htm,\
             patch("mbed_lstools.lstools_base.MbedLsToolsBase.mount_point_ready") as _mpr,\
             patch("mbed_os_tools.detect.platform_database.PlatformDatabase.get") as _get,\
             patch("mbed_os_tools.detect.lstools_base.listdir") as _listdir:
            _mpr.return_value = True
            _read_htm.return_value = (u'0241BEEFDEAD', {})
            _get.return_value = {
                'platform_name': 'foo_target'
            }
            _listdir.return_value = ['mbed.htm']
            to_check = self.base.list_mbeds()
            _read_htm.assert_called_once_with('dummy_mount_point')
            _get.assert_any_call('0241', device_type='daplink', verbose_data=True)
        self.assertEqual(len(to_check), 1)
        self.assertEqual(to_check[0]['target_id'], "0241BEEFDEAD")
        self.assertEqual(to_check[0]['platform_name'], 'foo_target')

    def test_list_mbeds_invalid_tid(self):
        self.base.return_value = [{'mount_point': 'dummy_mount_point',
                                   'target_id_usb_id': u'0240DEADBEEF',
                                   'serial_port': "dummy_serial_port"},
                                  {'mount_point': 'dummy_mount_point',
                                   'target_id_usb_id': "",
                                   'serial_port': 'not_valid'}]
        with patch("mbed_lstools.lstools_base.MbedLsToolsBase._read_htm_ids") as _read_htm,\
             patch("mbed_lstools.lstools_base.MbedLsToolsBase.mount_point_ready") as _mpr,\
             patch("mbed_os_tools.detect.platform_database.PlatformDatabase.get") as _get,\
             patch("mbed_os_tools.detect.lstools_base.listdir") as _listdir:
            _mpr.return_value = True
            _read_htm.side_effect = [(u'0241BEEFDEAD', {}), (None, {})]
            _get.return_value = {
                'platform_name': 'foo_target'
            }
            _listdir.return_value = ['mbed.htm']
            to_check = self.base.list_mbeds()
            _get.assert_any_call('0241', device_type='daplink', verbose_data=True)
        self.assertEqual(len(to_check), 2)
        self.assertEqual(to_check[0]['target_id'], "0241BEEFDEAD")
        self.assertEqual(to_check[0]['platform_name'], 'foo_target')
        self.assertEqual(to_check[1]['target_id'], "")
        self.assertEqual(to_check[1]['platform_name'], None)

    def test_list_mbeds_invalid_platform(self):
        self.base.return_value = [{'mount_point': 'dummy_mount_point',
                                   'target_id_usb_id': u'not_in_target_db',
                                   'serial_port': "dummy_serial_port"}]
        for qos in [FSInteraction.BeforeFilter, FSInteraction.AfterFilter]:
            with patch("mbed_lstools.lstools_base.MbedLsToolsBase._read_htm_ids") as _read_htm,\
                patch("mbed_lstools.lstools_base.MbedLsToolsBase.mount_point_ready") as _mpr,\
                patch("mbed_os_tools.detect.platform_database.PlatformDatabase.get") as _get,\
                patch("mbed_os_tools.detect.lstools_base.listdir") as _listdir:
                _mpr.return_value = True
                _read_htm.return_value = (u'not_in_target_db', {})
                _get.return_value = None
                _listdir.return_value = ['MBED.HTM']
                to_check = self.base.list_mbeds()
                _read_htm.assert_called_once_with('dummy_mount_point')
                _get.assert_any_call('not_', device_type='daplink', verbose_data=True)
            self.assertEqual(len(to_check), 1)
            self.assertEqual(to_check[0]['target_id'], "not_in_target_db")
            self.assertEqual(to_check[0]['platform_name'], None)

    def test_list_mbeds_unmount_mid_read(self):
        self.base.return_value = [{'mount_point': 'dummy_mount_point',
                                   'target_id_usb_id': u'0240DEADBEEF',
                                   'serial_port': "dummy_serial_port"}]
        with patch("mbed_lstools.lstools_base.MbedLsToolsBase.mount_point_ready") as _mpr,\
             patch("mbed_os_tools.detect.lstools_base.listdir") as _listdir:
            _mpr.return_value = True
            _listdir.side_effect = OSError
            to_check = self.base.list_mbeds()
        self.assertEqual(len(to_check), 0)

    def test_list_mbeds_read_mbed_htm_failure(self):
        def _test(mock):
            self.base.return_value = [{'mount_point': 'dummy_mount_point',
                                       'target_id_usb_id': u'0240DEADBEEF',
                                       'serial_port': "dummy_serial_port"}]
            with patch("mbed_lstools.lstools_base.MbedLsToolsBase.mount_point_ready") as _mpr,\
                 patch("mbed_os_tools.detect.lstools_base.listdir") as _listdir,\
                 patch('mbed_os_tools.detect.lstools_base.open', mock, create=True):
                _mpr.return_value = True
                _listdir.return_value = ['MBED.HTM', 'DETAILS.TXT']
                to_check = self.base.list_mbeds()
                mock.assert_called_once_with(os.path.join('dummy_mount_point', 'mbed.htm'), 'r')
                self.assertEqual(len(to_check), 0)

        m = mock_open()
        m.side_effect = OSError
        _test(m)

        m.reset_mock()
        m.side_effect = IOError
        _test(m)

    def test_list_mbeds_read_no_mbed_htm(self):
        self.base.return_value = [{'mount_point': 'dummy_mount_point',
                                   'target_id_usb_id': u'0240DEADBEEF',
                                   'serial_port': "dummy_serial_port"}]

        details_txt_contents = '''\
# DAPLink Firmware - see https://mbed.com/daplink
Unique ID: 0240000032044e4500257009997b00386781000097969900
HIC ID: 97969900
Auto Reset: 0
Automation allowed: 1
Overflow detection: 1
Daplink Mode: Interface
Interface Version: 0246
Bootloader Version: 0244
Git SHA: 0beabef8aa4b382809d79e98321ecf6a28936812
Local Mods: 0
USB Interfaces: MSD, CDC, HID
Bootloader CRC: 0xb92403e6
Interface CRC: 0x434eddd1
Remount count: 0
'''
        def _handle_open(*args, **kwargs):
            if args[0].lower() == os.path.join('dummy_mount_point', 'mbed.htm'):
                raise OSError("(mocked open) No such file or directory: 'mbed.htm'")
            else:
                return DEFAULT

        m = mock_open(read_data=details_txt_contents)
        with patch("mbed_lstools.lstools_base.MbedLsToolsBase.mount_point_ready") as _mpr,\
             patch("mbed_os_tools.detect.lstools_base.listdir") as _listdir,\
             patch('mbed_os_tools.detect.lstools_base.open', m, create=True) as mocked_open:
            mocked_open.side_effect = _handle_open
            _mpr.return_value = True
            _listdir.return_value = ['PRODINFO.HTM', 'DETAILS.TXT']
            to_check = self.base.list_mbeds()
            self.assertEqual(len(to_check), 1)
            m.assert_called_once_with(os.path.join('dummy_mount_point', 'DETAILS.TXT'), 'r')
            self.assertEqual(to_check[0]['target_id'], '0240000032044e4500257009997b00386781000097969900')

    def test_list_mbeds_read_details_txt_failure(self):
        def _test(mock):
            self.base.return_value = [{'mount_point': 'dummy_mount_point',
                                       'target_id_usb_id': u'0240DEADBEEF',
                                       'serial_port': "dummy_serial_port"}]
            with patch("mbed_lstools.lstools_base.MbedLsToolsBase.mount_point_ready") as _mpr,\
                 patch("mbed_os_tools.detect.lstools_base.listdir") as _listdir,\
                 patch("mbed_lstools.lstools_base.MbedLsToolsBase._update_device_from_htm") as _htm,\
                 patch('mbed_os_tools.detect.lstools_base.open', mock, create=True):
                _mpr.return_value = True
                _htm.side_effect = None
                _listdir.return_value = ['MBED.HTM', 'DETAILS.TXT']
                to_check = self.base.list_mbeds(read_details_txt=True)
                mock.assert_called_once_with(os.path.join('dummy_mount_point', 'DETAILS.TXT'), 'r')
                self.assertEqual(len(to_check), 0)

        m = mock_open()
        m.side_effect = OSError
        _test(m)

        m.reset_mock()
        m.side_effect = IOError
        _test(m)

    def test_list_mbeds_unmount_mid_read_list_unmounted(self):
        self.base.list_unmounted = True
        self.base.return_value = [{'mount_point': 'dummy_mount_point',
                                   'target_id_usb_id': u'0240DEADBEEF',
                                   'serial_port': "dummy_serial_port"}]
        with patch("mbed_lstools.lstools_base.MbedLsToolsBase.mount_point_ready") as _mpr,\
             patch("mbed_os_tools.detect.lstools_base.listdir") as _listdir:
            _mpr.return_value = True
            _listdir.side_effect = OSError
            to_check = self.base.list_mbeds()
        self.assertEqual(len(to_check), 1)
        self.assertEqual(to_check[0]['mount_point'], None)
        self.assertEqual(to_check[0]['device_type'], 'unknown')
        self.assertEqual(to_check[0]['platform_name'], 'K64F')

    def test_list_manufacture_ids(self):
        table_str = self.base.list_manufacture_ids()
        self.assertTrue(isinstance(table_str, basestring))

    def test_mock_manufacture_ids_default_multiple(self):
        # oper='+'
        for mid, platform_name in [('0341', 'TEST_PLATFORM_NAME_1'),
                                   ('0342', 'TEST_PLATFORM_NAME_2'),
                                   ('0343', 'TEST_PLATFORM_NAME_3')]:
            self.base.mock_manufacture_id(mid, platform_name)
            self.assertEqual(platform_name, self.base.plat_db.get(mid))

    def test_mock_manufacture_ids_minus(self):
        # oper='+'
        for mid, platform_name in [('0341', 'TEST_PLATFORM_NAME_1'),
                                   ('0342', 'TEST_PLATFORM_NAME_2'),
                                   ('0343', 'TEST_PLATFORM_NAME_3')]:
            self.base.mock_manufacture_id(mid, platform_name)
            self.assertEqual(platform_name, self.base.plat_db.get(mid))

        # oper='-'
        mock_ids = self.base.mock_manufacture_id('0342', '', oper='-')
        self.assertEqual('TEST_PLATFORM_NAME_1', self.base.plat_db.get("0341"))
        self.assertEqual(None, self.base.plat_db.get("0342"))
        self.assertEqual('TEST_PLATFORM_NAME_3', self.base.plat_db.get("0343"))

    def test_mock_manufacture_ids_star(self):
        # oper='+'
        for mid, platform_name in [('0341', 'TEST_PLATFORM_NAME_1'),
                                   ('0342', 'TEST_PLATFORM_NAME_2'),
                                   ('0343', 'TEST_PLATFORM_NAME_3')]:
            self.base.mock_manufacture_id(mid, platform_name)

            self.assertEqual(platform_name, self.base.plat_db.get(mid))

        # oper='-'
        self.base.mock_manufacture_id('*', '', oper='-')
        self.assertEqual(None, self.base.plat_db.get("0341"))
        self.assertEqual(None, self.base.plat_db.get("0342"))
        self.assertEqual(None, self.base.plat_db.get("0343"))

    def test_update_device_from_fs_mid_unmount(self):
        dummy_mount = 'dummy_mount'
        device = {
            'mount_point': dummy_mount
        }

        with patch("mbed_os_tools.detect.lstools_base.listdir") as _listdir:
            _listdir.side_effect = OSError
            self.base._update_device_from_fs(device, False)
            self.assertEqual(device['mount_point'], None)

    def test_detect_device_test(self):
        device_type = self.base._detect_device_type({
            'vendor_id': '0483'
        })
        self.assertEqual(device_type, 'stlink')

        device_type = self.base._detect_device_type({
            'vendor_id': '0d28'
        })
        self.assertEqual(device_type, 'daplink')

        device_type = self.base._detect_device_type({
            'vendor_id': '1366'
        })
        self.assertEqual(device_type, 'jlink')

    def test_device_type_unmounted(self):
        self.base.list_unmounted = True
        self.base.return_value = [{'mount_point': None,
                                   'target_id_usb_id': u'0240DEADBEEF',
                                   'serial_port': "dummy_serial_port",
                                   'vendor_id': '0d28',
                                   'product_id': '0204'}]
        with patch("mbed_os_tools.detect.platform_database.PlatformDatabase.get") as _get,\
             patch("mbed_os_tools.detect.lstools_base.listdir") as _listdir:
            _get.return_value = {
                'platform_name': 'foo_target'
            }
            to_check = self.base.list_mbeds()
            #_get.assert_any_call('0240', device_type='daplink', verbose_data=True)
        self.assertEqual(len(to_check), 1)
        self.assertEqual(to_check[0]['target_id'], "0240DEADBEEF")
        self.assertEqual(to_check[0]['platform_name'], 'foo_target')
        self.assertEqual(to_check[0]['device_type'], 'daplink')

    def test_update_device_details_jlink(self):
        jlink_html_contents = ('<html><head><meta http-equiv="refresh" '
                               'content="0; url=http://www.nxp.com/FRDM-KL27Z"/>'
                               '<title>NXP Product Page</title></head><body></body></html>')
        _open = mock_open(read_data=jlink_html_contents)
        dummy_mount_point = 'dummy'
        base_device = {
            'mount_point': dummy_mount_point
        }

        with patch('mbed_os_tools.detect.lstools_base.open', _open, create=True):
            device = deepcopy(base_device)
            device['directory_entries'] = ['Board.html', 'User Guide.html']
            self.base._update_device_details_jlink(device, False)
            self.assertEqual(device['url'], 'http://www.nxp.com/FRDM-KL27Z')
            self.assertEqual(device['platform_name'], 'KL27Z')
            _open.assert_called_once_with(os.path.join(dummy_mount_point, 'Board.html'), 'r')

            _open.reset_mock()

            device = deepcopy(base_device)
            device['directory_entries'] = ['User Guide.html']
            self.base._update_device_details_jlink(device, False)
            self.assertEqual(device['url'], 'http://www.nxp.com/FRDM-KL27Z')
            self.assertEqual(device['platform_name'], 'KL27Z')
            _open.assert_called_once_with(os.path.join(dummy_mount_point, 'User Guide.html'), 'r')

            _open.reset_mock()

            device = deepcopy(base_device)
            device['directory_entries'] = ['unhelpful_file.html']
            self.base._update_device_details_jlink(device, False)
            _open.assert_not_called()

    def test_fs_never(self):
        device = {
            'target_id_usb_id': '024075309420ABCE',
            'mount_point': 'invalid_mount_point',
            'serial_port': 'invalid_serial_port'
        }
        self.base.return_value = [device]
        with patch("mbed_lstools.lstools_base.MbedLsToolsBase._update_device_from_fs") as _up_fs,\
             patch("mbed_lstools.lstools_base.MbedLsToolsBase.mount_point_ready") as mount_point_ready:
            mount_point_ready.return_value = True

            filter = None
            ret = self.base.list_mbeds(FSInteraction.Never, filter, read_details_txt=False)
            ret_with_details = self.base.list_mbeds(FSInteraction.Never, filter, read_details_txt=True)
            self.assertIsNotNone(ret[0])
            self.assertIsNotNone(ret_with_details[0])
            self.assertEqual(ret[0]['target_id'], ret[0]['target_id_usb_id'])
            self.assertEqual(ret[0]['platform_name'], "K64F")
            self.assertEqual(ret[0], ret_with_details[0])
            _up_fs.assert_not_called()

            filter_in = lambda m: m['platform_name'] == 'K64F'
            ret = self.base.list_mbeds(FSInteraction.Never, filter_in, read_details_txt=False)
            ret_with_details = self.base.list_mbeds(FSInteraction.Never, filter_in, read_details_txt=True)
            self.assertIsNotNone(ret[0])
            self.assertIsNotNone(ret_with_details[0])
            self.assertEqual(ret[0]['target_id'], ret[0]['target_id_usb_id'])
            self.assertEqual(ret[0]['platform_name'], "K64F")
            self.assertEqual(ret[0], ret_with_details[0])
            _up_fs.assert_not_called()

            filter_out = lambda m: m['platform_name'] != 'K64F'
            ret = self.base.list_mbeds(FSInteraction.Never, filter_out, read_details_txt=False)
            ret_with_details = self.base.list_mbeds(FSInteraction.Never, filter_out, read_details_txt=True)
            _up_fs.assert_not_called()
            self.assertEqual(ret, [])
            self.assertEqual(ret, ret_with_details)
            _up_fs.assert_not_called()

    def test_fs_after(self):
        device = {
            'target_id_usb_id': '024075309420ABCE',
            'mount_point': 'invalid_mount_point',
            'serial_port': 'invalid_serial_port'
        }
        with patch("mbed_lstools.lstools_base.MbedLsToolsBase._read_htm_ids") as _read_htm,\
             patch("mbed_lstools.lstools_base.MbedLsToolsBase._details_txt") as _up_details,\
             patch("mbed_lstools.lstools_base.MbedLsToolsBase.mount_point_ready") as mount_point_ready,\
             patch("mbed_os_tools.detect.lstools_base.listdir") as _listdir:
            new_device_id = "00017531642046"
            _read_htm.return_value = (new_device_id, {})
            _listdir.return_value = ['mbed.htm', 'details.txt']
            _up_details.return_value = {
                'automation_allowed': '0'
            }
            mount_point_ready.return_value = True

            filter = None
            self.base.return_value = [deepcopy(device)]
            ret = self.base.list_mbeds(FSInteraction.AfterFilter, filter, False, False)
            _up_details.assert_not_called()
            self.base.return_value = [deepcopy(device)]
            ret_with_details = self.base.list_mbeds(FSInteraction.AfterFilter, filter, False, True)

            self.assertIsNotNone(ret[0])
            self.assertIsNotNone(ret_with_details[0])
            self.assertEqual(ret[0]['target_id'], new_device_id)
            self.assertEqual(ret_with_details[0]['daplink_automation_allowed'], '0')
            self.assertDictContainsSubset(ret[0], ret_with_details[0])
            _read_htm.assert_called_with(device['mount_point'])
            _up_details.assert_called_with(device['mount_point'])

            _read_htm.reset_mock()
            _up_details.reset_mock()

            filter_in = lambda m: m['target_id'] == device['target_id_usb_id']
            filter_details = lambda m: m.get('daplink_automation_allowed', None) == '0'

            self.base.return_value = [deepcopy(device)]
            ret = self.base.list_mbeds(
                FSInteraction.AfterFilter, filter_in, False, False)
            self.base.return_value = [deepcopy(device)]
            ret_with_details = self.base.list_mbeds(
                FSInteraction.AfterFilter, filter_details, False, True)

            self.assertIsNotNone(ret[0])
            self.assertEqual(ret_with_details, [])
            self.assertEqual(ret[0]['target_id'], new_device_id)
            _read_htm.assert_called_with(device['mount_point'])
            _up_details.assert_not_called()

            _read_htm.reset_mock()
            _up_details.reset_mock()

            filter_out = lambda m: m['target_id'] == new_device_id

            self.base.return_value = [deepcopy(device)]
            ret = self.base.list_mbeds(
                FSInteraction.AfterFilter, filter_out, False, False)
            self.base.return_value = [deepcopy(device)]
            ret_with_details = self.base.list_mbeds(
                FSInteraction.AfterFilter, filter_out, False, True)

            self.assertEqual(ret, [])
            self.assertEqual(ret_with_details, [])
            _read_htm.assert_not_called()
            _up_details.assert_not_called()

    def test_get_supported_platforms(self):
        supported_platforms = self.base.get_supported_platforms()
        self.assertTrue(isinstance(supported_platforms, dict))
        self.assertEqual(supported_platforms['0240'], 'K64F')

    def test_fs_before(self):
        device = {
            'target_id_usb_id': '024075309420ABCE',
            'mount_point': 'invalid_mount_point',
            'serial_port': 'invalid_serial_port'
        }
        with patch("mbed_lstools.lstools_base.MbedLsToolsBase._read_htm_ids") as _read_htm,\
             patch("mbed_lstools.lstools_base.MbedLsToolsBase._details_txt") as _up_details,\
             patch("mbed_lstools.lstools_base.MbedLsToolsBase.mount_point_ready") as mount_point_ready,\
             patch("mbed_os_tools.detect.lstools_base.listdir") as _listdir:
            new_device_id = u'00017575430420'
            _read_htm.return_value = (new_device_id, {})
            _listdir.return_value = ['mbed.htm', 'details.txt']
            _up_details.return_value = {
                'automation_allowed': '0'
            }
            mount_point_ready.return_value = True

            filter = None
            self.base.return_value = [deepcopy(device)]
            ret = self.base.list_mbeds(
                FSInteraction.BeforeFilter, filter, False, False)
            _up_details.assert_not_called()

            self.base.return_value = [deepcopy(device)]
            ret_with_details = self.base.list_mbeds(
                FSInteraction.BeforeFilter, filter, False, True)
            self.assertIsNotNone(ret[0])
            self.assertIsNotNone(ret_with_details[0])
            self.assertEqual(ret[0]['target_id'], new_device_id)
            self.assertEqual(ret_with_details[0]['daplink_automation_allowed'], '0')
            self.assertDictContainsSubset(ret[0], ret_with_details[0])
            _read_htm.assert_called_with(device['mount_point'])
            _up_details.assert_called_with(device['mount_point'])

            _read_htm.reset_mock()
            _up_details.reset_mock()

            filter_in = lambda m: m['target_id'] == '00017575430420'
            filter_in_details = lambda m: m['daplink_automation_allowed'] == '0'
            self.base.return_value = [deepcopy(device)]
            ret = self.base.list_mbeds(
                FSInteraction.BeforeFilter, filter_in, False, False)
            _up_details.assert_not_called()

            self.base.return_value = [deepcopy(device)]
            ret_with_details = self.base.list_mbeds(
                FSInteraction.BeforeFilter, filter_in_details, False, True)

            self.assertIsNotNone(ret[0])
            self.assertIsNotNone(ret_with_details[0])
            self.assertEqual(ret[0]['target_id'], new_device_id)
            self.assertEqual(ret_with_details[0]['daplink_automation_allowed'], '0')
            self.assertDictContainsSubset(ret[0], ret_with_details[0])
            _read_htm.assert_called_with(device['mount_point'])
            _up_details.assert_called_with(device['mount_point'])

            _read_htm.reset_mock()
            _up_details.reset_mock()

            filter_out = lambda m: m['target_id'] == '024075309420ABCE'
            filter_out_details = lambda m: m['daplink_automation_allowed'] == '1'
            ret = self.base.list_mbeds(
                FSInteraction.BeforeFilter, filter_out, False, False)
            _up_details.assert_not_called()

            self.base.return_value = [deepcopy(device)]
            ret_with_details = self.base.list_mbeds(
                FSInteraction.BeforeFilter, filter_out_details, False, True)

            self.assertEqual(ret, [])
            self.assertEqual(ret_with_details, [])
            _read_htm.assert_called_with(device['mount_point'])

class RetargetTestCase(unittest.TestCase):
    """ Test cases that makes use of retargetting
    """

    def setUp(self):
        retarget_data = {
            '0240DEADBEEF': {
                'serial_port' : 'valid'
            }
        }

        _open = mock_open(read_data=json.dumps(retarget_data))

        with patch('os.path.isfile') as _isfile,\
             patch('mbed_os_tools.detect.lstools_base.isfile') as _isfile,\
             patch('mbed_os_tools.detect.lstools_base.open', _open, create=True):
            _isfile.return_value = True
            self.base = DummyLsTools()
            _open.assert_called()

    def tearDown(self):
        pass

    def test_list_mbeds_valid_platform(self):
        self.base.return_value = [{'mount_point': 'dummy_mount_point',
                                   'target_id_usb_id': u'0240DEADBEEF',
                                   'serial_port': None}]
        with patch('mbed_lstools.lstools_base.MbedLsToolsBase._read_htm_ids') as _read_htm,\
             patch('mbed_lstools.lstools_base.MbedLsToolsBase.mount_point_ready') as _mpr,\
             patch('mbed_os_tools.detect.platform_database.PlatformDatabase.get') as _get,\
             patch("mbed_os_tools.detect.lstools_base.listdir") as _listdir:
            _mpr.return_value = True
            _read_htm.return_value = (u'0240DEADBEEF', {})
            _get.return_value = {
                'platform_name': 'foo_target'
            }
            _listdir.return_value = ['mbed.htm']
            to_check = self.base.list_mbeds()
        self.assertEqual(len(to_check), 1)
        self.assertEqual(to_check[0]['serial_port'], 'valid')

if __name__ == '__main__':
    unittest.main()



"""
mbed SDK
Copyright (c) 2011-2018 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import sys
import plistlib
from mock import MagicMock, patch
from six import BytesIO

from mbed_lstools.darwin import MbedLsToolsDarwin

class DarwinTestCase(unittest.TestCase):
    """Tests for the Darwin port
    """

    def setUp(self):
        with patch("platform.mac_ver") as _pv:
            _pv.return_value = ["10.2.2"]
            self.darwin = MbedLsToolsDarwin()

    def tearDown(self):
        pass

    def test_a_k64f(self):
        disks = {
            'AllDisks': ['disk0', 'disk0s1', 'disk0s2', 'disk0s3', 'disk1', 'disk2'],
            'AllDisksAndPartitions': [{ 'Content': 'GUID_partition_scheme',
                  'DeviceIdentifier': 'disk0',
                  'Partitions': [
                      { 'Content': 'EFI',
                        'DeviceIdentifier': 'disk0s1',
                        'DiskUUID': 'nope',
                        'Size': 209715200,
                        'VolumeName': 'EFI',
                        'VolumeUUID': 'nu-uh'},
                      { 'Content': 'Apple_CoreStorage',
                        'DeviceIdentifier': 'disk0s2',
                        'DiskUUID': 'nodda',
                        'Size': 250006216704},
                      { 'Content': 'Apple_Boot',
                        'DeviceIdentifier': 'disk0s3',
                        'DiskUUID': 'no soup for you!',
                        'Size': 650002432,
                        'VolumeName': 'Recovery HD',
                        'VolumeUUID': 'Id rather not'}],
                  'Size': 251000193024},
                { 'Content': 'Apple_HFS',
                  'DeviceIdentifier': 'disk1',
                  'MountPoint': '/',
                  'Size': 249653772288,
                  'VolumeName': 'Mac HD'},
                { 'Content': '',
                  'DeviceIdentifier': 'disk2',
                  'MountPoint': '/Volumes/DAPLINK',
                  'Size': 67174400,
                  'VolumeName': 'DAPLINK'}],
            'VolumesFromDisks': ['Mac HD', 'DAPLINK'],
            'WholeDisks': ['disk0', 'disk1', 'disk2']
        }
        usb_tree = [{
            'IORegistryEntryName': 'DAPLink CMSIS-DAP',
            'USB Serial Number': '0240000034544e45003a00048e3800525a91000097969900',
            'idProduct': 516,
            'idVendor': 3368,
            'IORegistryEntryChildren': [
                {'IORegistryEntryName': 'AppleUSBHostLegacyClient'},
                {'IORegistryEntryName': 'AppleUSBHostCompositeDevice'},
                {'IORegistryEntryName': 'USB_MSC',
                 'idProduct': 516,
                 'idVendor': 3368,
                 'IORegistryEntryChildren': [
                    {'IORegistryEntryName': 'IOUSBMassStorageInterfaceNub',
                     'idProduct': 516,
                     'idVendor': 3368,
                     'IORegistryEntryChildren': [
                         {'IORegistryEntryName': 'IOUSBMassStorageDriverNub',
                          'idProduct': 516,
                          'idVendor': 3368,
                          'IORegistryEntryChildren': [
                             {'IORegistryEntryName': 'IOUSBMassStorageDriver',
                              'IORegistryEntryChildren': [
                                 {'IORegistryEntryName': 'IOSCSILogicalUnitNub',
                                  'IORegistryEntryChildren': [
                                     {'IORegistryEntryName': 'IOSCSIPeripheralDeviceType00',
                                      'IORegistryEntryChildren': [
                                         {'IORegistryEntryName': 'IOBlockStorageServices',
                                          'IORegistryEntryChildren': [
                                             {'IORegistryEntryName': 'IOBlockStorageDriver',
                                              'IORegistryEntryChildren': [
                                                 {'BSD Name': 'disk2',
                                                  'IORegistryEntryName': 'MBED VFS Media',
                                                  'IORegistryEntryChildren': [
                                                      {'IORegistryEntryName': 'IOMediaBSDClient'}],
                                                  }],
                                              }],
                                         }],
                                     }],
                                 }],
                             }],
                         }],
                    }],
                 },
                {'IORegistryEntryName': 'CMSIS-DAP',
                 'idProduct': 516,
                 'idVendor': 3368,
                 'IORegistryEntryChildren': [
                    {'IORegistryEntryName': 'IOUSBHostHIDDevice',
                     'IORegistryEntryChildren': [
                        {'IORegistryEntryName': 'IOHIDInterface'},
                        {'IORegistryEntryName': 'IOHIDLibUserClient'},
                        {'IORegistryEntryName': 'IOHIDLibUserClient'}],
                     }],
                  },
                {'IORegistryEntryName': 'mbed Serial Port',
                 'idProduct': 516,
                 'idVendor': 3368,
                 'IORegistryEntryChildren': [
                     { 'IORegistryEntryName': 'AppleUSBACMControl'}],
                },
                {'IORegistryEntryName': 'mbed Serial Port',
                 'idProduct': 516,
                 'idVendor': 3368,
                 'IORegistryEntryChildren': [
                    {'IORegistryEntryName': 'AppleUSBACMData',
                     'idProduct': 516,
                     'idVendor': 3368,
                     'IORegistryEntryChildren': [
                        {'IORegistryEntryName': 'IOModemSerialStreamSync',
                         'IORegistryEntryChildren': [
                             {'IODialinDevice': '/dev/tty.usbmodem1422',
                              'IORegistryEntryName': 'IOSerialBSDClient'}],
                        }],
                     }],
                 }],
            }
        ]

        with patch("subprocess.Popen") as _popen:
            def do_popen(command, *args, **kwargs):
                to_ret = MagicMock()
                to_ret.wait.return_value = 0
                to_ret.stdout = BytesIO()
                plistlib.writePlist(
                    {'diskutil': disks,
                     'ioreg': usb_tree}[command[0]],
                    to_ret.stdout)
                to_ret.stdout.seek(0)
                to_ret.communicate.return_value = (to_ret.stdout.getvalue(), "")
                return to_ret
            _popen.side_effect = do_popen
            candidates = self.darwin.find_candidates()
        self.assertIn({'mount_point': '/Volumes/DAPLINK',
                       'serial_port': '/dev/tty.usbmodem1422',
                       'target_id_usb_id': '0240000034544e45003a00048e3800525a91000097969900',
                       'vendor_id': '0d28',
                       'product_id': '0204'},
                      candidates)
